﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using Windows.Networking.Proximity;
using Windows.Networking.Sockets;
using Windows.Storage.Streams;

namespace _03_TapSend {
    public partial class MainPage : PhoneApplicationPage {
        private string _command;
        DataWriter _dataWriter;

        public MainPage() {
            InitializeComponent();
        }

        /// <summary>
        /// Send the file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTapSend_Click(object sender, RoutedEventArgs e) {
            _command = ((Button)sender).CommandParameter.ToString();
            PeerFinder.TriggeredConnectionStateChanged += TriggeredConnectionStateChanged;
            PeerFinder.Start();
        }

        /// <summary>
        /// Handle the connection state changes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private async void TriggeredConnectionStateChanged(object sender, TriggeredConnectionStateChangedEventArgs args) {
            if (args.State == TriggeredConnectState.PeerFound) {
                WriteLog("Peer found. Keep your devices together until the connection is established.");
            }

            if (args.State == TriggeredConnectState.Completed) {
                WriteLog("Connected!");
                switch (_command) {
                    case "send":
                        WriteLog("Sending picture");
                        SendPicture(args.Socket);
                        break;
                    case "receive":
                        WriteLog("Receiving picture");
                        await ReceivePicture(args.Socket);
                        Dispatcher.BeginInvoke(() =>
                            {
                                DispalyPicture("image1.jpg");
                            });
                        break;
                }
            }
            //failure to connect
            if (args.State == TriggeredConnectState.Failed && args.State == TriggeredConnectState.Canceled) {
                WriteLog("Failed to connect!  Please try to pair the devices again\n");
            }
        }

        /// <summary>
        /// Send the picture
        /// </summary>
        /// <param name="socket"></param>
        private async void SendPicture(StreamSocket socket) {
            // Create DataWriter for writing to peer.
            _dataWriter = new DataWriter(socket.OutputStream);
            // read song from Isolated Storage and send it
            using (var fileStream = new IsolatedStorageFileStream("image1.jpg", FileMode.Open, FileAccess.Read, IsolatedStorageFile.GetUserStoreForApplication())) {
                WriteLog("Sending song data");
                byte[] buffer = new byte[1024];
                int bytesRead;
                int readCount = 0;
                while ((bytesRead = fileStream.Read(buffer, 0, buffer.Length)) != 0) {
                    readCount += bytesRead;
                    //size of the packet
                    _dataWriter.WriteInt32(bytesRead);
                    //packet data sent
                    _dataWriter.WriteBytes(buffer);
                    try {
                        await _dataWriter.StoreAsync();
                        WriteProgressLog("Sending file; total bytes sent:" + readCount.ToString());
                    } catch (Exception e) {
                        WriteLog("Send error: " + e.Message);
                    }
                }
                WriteProgressLog("Completed sending song data; total bytes sent:" + readCount.ToString());
            }
        }

        private async Task ReceivePicture(StreamSocket socket) {
            WriteLog("Receiving song data...");
            try {
                UInt32 totalBytesRead = 0;
                UInt32 initialLength = 4; //for the int @ the front of every packet that denotes size of the packet coming
                var dr = new DataReader(socket.InputStream);
                UInt32 packetLength = 0;

                //generate song filename _songFilename
                string imagename = "image1.jpg";
                IfFileExistsInIsoStoreDeleteIt(imagename);

                var fileStream = new IsolatedStorageFileStream(imagename, FileMode.CreateNew, IsolatedStorageFile.GetUserStoreForApplication());
                //await dr.LoadAsync()
                while ((packetLength = await GetNextPacketSize(initialLength, dr)) > 0) {
                    //read buffer of that size
                    if (packetLength > 0) {
                        byte[] buffer = new byte[packetLength];
                        await dr.LoadAsync(packetLength);
                        dr.ReadBytes(buffer);
                        //write to iso store song file
                        fileStream.Write(buffer, 0, (int)packetLength);
                    }

                    totalBytesRead += packetLength;
                    WriteProgressLog("Receiving. Total bytes received:" + totalBytesRead.ToString());
                }

                WriteLog("Completed receiving. Total bytes received:" + totalBytesRead.ToString());
                fileStream.Close();


            } catch (Exception e) {
                WriteLog("Error: " + e.Message + "\n");
            }
        }

        /// <summary>
        /// Display the picture
        /// </summary>
        /// <param name="fileName"></param>
        private void DispalyPicture(string fileName) {
            BitmapImage bi = new BitmapImage();
            using (IsolatedStorageFile myIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication()) {
                using (IsolatedStorageFileStream fileStream = myIsolatedStorage.OpenFile(fileName, FileMode.Open, FileAccess.Read)) {
                    bi.SetSource(fileStream);
                }
            }
            photo.Source = bi;
        }


        private void btnTakePhoto_Click(object sender, RoutedEventArgs e) {
            CameraCaptureTask photoCameraCapture = new CameraCaptureTask();
            photoCameraCapture.Completed += new EventHandler<PhotoResult>(photoCameraCapture_Completed);
            photoCameraCapture.Show();
        }

        void photoCameraCapture_Completed(object sender, PhotoResult e) {
            if (e.TaskResult == TaskResult.OK) {
                SaveToIsolatedStorage(e.ChosenPhoto, "image1.jpg");
                WriteLog("Image has been saved in the isolated storage successfully!!!");
                var image = new BitmapImage();
                image.SetSource(e.ChosenPhoto);
                photo.Source = image;
                btnTapSend.IsEnabled = true;

            }
        }

        private void SaveToIsolatedStorage(Stream imageStream, string fileName) {
            using (IsolatedStorageFile myIsolatedStorage = IsolatedStorageFile.GetUserStoreForApplication()) {
                if (myIsolatedStorage.FileExists(fileName)) {
                    myIsolatedStorage.DeleteFile(fileName);
                }

                IsolatedStorageFileStream fileStream = myIsolatedStorage.CreateFile(fileName);
                imageStream.CopyTo(fileStream);
                fileStream.Close();

            }
        }

        private static void IfFileExistsInIsoStoreDeleteIt(string songFileName) {
            // If file already exists delete it
            using (var store = IsolatedStorageFile.GetUserStoreForApplication()) {
                if (store.FileExists(songFileName))
                    store.DeleteFile(songFileName);
            }
        }

        private static async Task<UInt32> GetNextPacketSize(uint initialLength, DataReader dr) {
            await dr.LoadAsync(initialLength);
            UInt32 packetLength = dr.ReadUInt32();
            if (packetLength > 1024) {
                return 0;
            }
            return packetLength;
        }

        private void WriteLog(string text) {
            Dispatcher.BeginInvoke(() =>
            {
                tbxLog.Text += text;
                tbxLog.Text += "\r\n";
            });
        }

        private void WriteProgressLog(string text) {
            Dispatcher.BeginInvoke(() =>
            {
                tbxProgessLog.Text = text;
            });
        }
    }
}